import {NextFunction, Request, Response} from 'express';
import session from 'express-session';
import {AddressInfo} from 'net';
import passport from 'passport';
import {Strategy as GoogleStrategy} from 'passport-google-oauth20';
import initRoutes from '../routes';
import {logger, reportError, sentry} from './utils';
import {db, User} from '../models';
import branchResolverMiddleware from '../middleware/branch-resolver';
import userResolverMiddleware from '../middleware/user-resolver';

import bodyParser = require('body-parser');
import cookieParser = require('cookie-parser');
import express = require('express');
import http = require('http');
import cors = require('cors');

const KnexSessionStore = require('connect-session-knex')(session);
const uuidv4: any = require('uuid/v4');

const appHost = process.env.APP_HOST || 'http://dev.scpdb.org';

const sessionSecret = process.env.SESSION_SECRET;
if (!sessionSecret) {
    throw new Error('Session secret is required');
}
const googleClientId = process.env.GOOGLE_CLIENT_ID;
const googleClientSecret = process.env.GOOGLE_CLIENT_SECRET;
if (!googleClientId || !googleClientSecret) {
    throw new Error('Google client id and secret are required');
}

passport.use(new GoogleStrategy({
    clientID: googleClientId,
    clientSecret: googleClientSecret,
    callbackURL: `${appHost}/auth/google/callback`,
},
(accessToken, refreshToken, profile, cb) => {
    User.findOrCreateByGoogleId(profile.id)
        .then((id: string) => {
            return cb(undefined, {id});
        })
        .catch((error) => {
            return cb(error, undefined);
        });
}));

passport.serializeUser((user, done) => {
    done(null, user);
});

passport.deserializeUser((user, done) => {
    done(null, user);
});

const CORS_WHITELIST = [
    'https://m.scpfoundation.net',
    'http://dev.scpdb.org',
    'https://scpdev.resure.net',
];

const httpLogger = require('pino-http')({
    logger,
    genReqId: uuidv4,
    serializers: {
        req: (req: Request) => {
            return {
                id: req.id,
                method: req.method,
                url: req.url,
                headers: req.headers,
            };
        },
        res: (res: Response) => {
            return {
                statusCode: res.statusCode,
                header: res.header,
            };
        },
    },
});

export default function generateServer(): (port: string | number) => void {
    const app = express();
    const router = express.Router(); // eslint-disable-line new-cap

    app.disable('x-powered-by');
    app.use(httpLogger);
    app.use(sentry.requestHandler());
    app.use(bodyParser.json());
    app.use(cors({
        origin: CORS_WHITELIST,
    }));

    app.use(cookieParser());

    app.use(session({
        name: 'scp_session',
        cookie: {
            maxAge: 1000 * 60 * 60 * 24 * 30 * 2, // 2 months
        },
        secret: sessionSecret as string,
        store: new KnexSessionStore({knex: db}),
        resave: false,
        saveUninitialized: false,
    }));
    app.use(passport.initialize());
    app.use(passport.session());

    app.use(branchResolverMiddleware);
    app.use(userResolverMiddleware);

    app.get('/auth/google', (req, res) => {
        if (req.query.returnPath && req.query.returnPath.startsWith('/')) {
      req.session!.returnPath = req.query.returnPath;
        }
        passport.authenticate('google', {scope: ['profile']})(req, res);
    });

    app.get('/auth/google/callback',
        passport.authenticate('google', {failureRedirect: '/'}),
        (req, res) => {
            res.redirect(req.session!.returnPath || '/');
        });

    app.get('/auth/logout', (req, res) => {
        req.logout();
        res.redirect('/');
    });

    app.get('/auth/whoami', (req, res) => {
        if (req.userInfo) {
            const {id, username, wikidotUid} = req.userInfo;
            res.send({id, username, wikidotUid});
        } else {
            res.send({id: null});
        }
    });

    initRoutes(app);
    app.use('/', router);

    app.use(sentry.errorHandler());

    app.use((err: Error, req: Request, res: Response, next: NextFunction) => {
        if (req.log) {
            req.log.error(err, 'Unhandled error');
        } else {
            reportError(err, {message: 'Unhandled error'});
        }
        res.status(500).send('Internal server error');
    });

    return function listen(port: string | number): void {
        const server = http.createServer(app);
        server.on('listening', () => {
            logger.info(`Express server is listening on port ${(server.address() as AddressInfo).port}`);
        });
        server.on('error', (error) => {
            if ((error as any).syscall !== 'listen') {
                throw error;
            }
            switch ((error as any).code) {
            case 'EACCES':
                logger.error(`Port ${port} requires elevated privileges`);
                process.exit(1);
                break;
            case 'EADDRINUSE':
                logger.error(`Port ${port} is already in use`);
                process.exit(1);
                break;
            default:
                throw error;
            }
        });
        server.listen(port);
    };
}
